#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# expIconicMemory.py
#
# created 13-03-2020
# last mod 20-03-2020 KN
#

import pandas as pd
import string
import time
import codecs
from psychopy import core, visual, event

def drawStim():
    u1TS.draw()
    u2TS.draw()
    u3TS.draw()
    u4TS.draw()
    m1TS.draw()
    m2TS.draw()
    m3TS.draw()
    m4TS.draw()
    l1TS.draw()
    l2TS.draw()
    l3TS.draw()
    l4TS.draw()
    win.flip()

def updateStim(item):
    u1TS.setText(item[0])
    u2TS.setText(item[1])
    u3TS.setText(item[2])
    u4TS.setText(item[3])
    m1TS.setText(item[4])
    m2TS.setText(item[5])
    m3TS.setText(item[6])
    m4TS.setText(item[7])
    l1TS.setText(item[8])
    l2TS.setText(item[9])
    l3TS.setText(item[10])
    l4TS.setText(item[11])

def updateResp(position, captured_string):
    position.text = captured_string
    drawStim()

if __name__ == '__main__':
    # Collect subject info
    expinfo = dict( id = input('Sechsstellige Vp-ID (z.B. KN1503): '),
                   age = input('Vp-Alter (z.B. 23): '),
                gender = input('Vp-Geschlecht (w/m): '),
               session = 'ses01.txt', #input('Session file: '),
                tstamp = time.strftime('%Y%m%d_%H%M', time.localtime()) )

    win = visual.Window([920,680], color='black', units='norm')
    timer = core.Clock()  # keep track of time during each trial

    # text height
    th = .15

    # stimulus presentation time (50 ms)
    stim_duration = 0.05
    isi           = 0.5 

    # initialise stimulus
    u1TS = visual.TextStim(win, text='', pos=(-.45, .3), height=th)
    u2TS = visual.TextStim(win, text='', pos=(-.15, .3), height=th)
    u3TS = visual.TextStim(win, text='', pos=( .15, .3), height=th)
    u4TS = visual.TextStim(win, text='', pos=( .45, .3), height=th)
    m1TS = visual.TextStim(win, text='', pos=(-.45,  0), height=th)
    m2TS = visual.TextStim(win, text='', pos=(-.15,  0), height=th)
    m3TS = visual.TextStim(win, text='', pos=( .15,  0), height=th)
    m4TS = visual.TextStim(win, text='', pos=( .45,  0), height=th)
    l1TS = visual.TextStim(win, text='', pos=(-.45,-.3), height=th)
    l2TS = visual.TextStim(win, text='', pos=(-.15,-.3), height=th)
    l3TS = visual.TextStim(win, text='', pos=( .15,-.3), height=th)
    l4TS = visual.TextStim(win, text='', pos=( .45,-.3), height=th)

    instrTS = visual.TextStim(win,
          u'In diesem Experiment bekommen Sie für einen Augenblick eine Tabelle '
          u'mit Buchstaben und Zahlen präsentiert.\nIm Anschluss sehen Sie eine '
          u'leere Tabelle oder Zeile. Geben Sie bitte alle Zeichen an die Sie '
          u'sich erinnern in die entsprechende Zelle ein und drücken danach '
          u'die Enter-Taste.\n'
          u'Zum Starten des Experiments bitte die Leertaste drücken.',
          height=.6*th, wrapWidth=1.5)

    instrTS.draw()
    win.flip()
    key = None
    while key not in ['space', 'escape']:
        key = event.waitKeys()[0]
        if key == 'escape':
            core.quit()
    win.flip()
    core.wait(1)

    with codecs.open('data/subj' + expinfo['id'] + '-' + expinfo['tstamp'] + '.txt',
                     'w', encoding='utf-8') as datfile:
        datfile.write('# ' + str(expinfo) + '\n')
        datfile.write('report;stim;resp;rt\n')

        data = pd.read_table('session/' + expinfo['session'], sep=';', header=None,
                dtype=pd.StringDtype())
        
        for i in data.index:
            item = data.iloc[i, 0:12]
            report = data.iloc[i, 12]
            response = ''

            # report condition
            if report == 'full':
                cue = pd.Series(['_']*12)
                positionList = [u1TS, u2TS, u3TS, u4TS, m1TS, m2TS, m3TS, m4TS,
                        l1TS, l2TS, l3TS, l4TS]
            elif report == 'upperPartial':
                cue = pd.Series(['_']*4 + ['']*8)
                positionList = [u1TS, u2TS, u3TS, u4TS]
            elif report == 'middlePartial':
                cue = pd.Series(['']*4 + ['_']*4 + ['']*4)
                positionList = [m1TS, m2TS, m3TS, m4TS]
            elif report == 'lowerPartial':
                cue = pd.Series(['']*8 + ['_']*4)
                positionList = [l1TS, l2TS, l3TS, l4TS]

            # present item
            updateStim(item)
            drawStim()
            core.wait(stim_duration)
            win.flip()

            # present cue
            updateStim(cue)
            drawStim()

            # get response for each position
            timer.reset()
            for position in positionList:
                captured_string = ''
                while True:
                    key = event.waitKeys()[0]
                    if key == 'return':
                        if captured_string == '':
                            pass
                        else:  # response finished
                            break
                    elif key == 'escape':
                        core.quit()
                    elif key in ['delete', 'backspace']:
                        captured_string = captured_string[:-1]  # delete last character
                        updateResp(position, captured_string)
                    elif key in list(string.ascii_lowercase) + list(string.ascii_uppercase) + [str(i) for i in range(10)]:
                        captured_string = key.capitalize()
                        updateResp(position, captured_string)
                    else:  # ignore any other input
                        pass
                response += captured_string

            # save response data
            datfile.write('%1s;%1s;%1s;%1.3f\n' %
                  (report, item.str.cat(), response, timer.getTime()))
            core.wait(isi)

    visual.TextStim(win, 'Ende - Vielen Dank').draw()
    win.flip()
    key = None
    while key not in ['space', 'escape']:
        key = event.waitKeys()[0]
